/*
  
  File cddata.c - routines for 2048 byte frames for cdfs

  
  Copyright (c) 1999,2000 by Michiel Ronsse 
  
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  
  
*/


#include "cdfs.h"


/***************************************************************************/


int cdfs_bmap(struct inode *inode, int block){
  int result;
  cd * this_cd = inode->i_sb->u.generic_sbp;
  int track_no = inode->i_ino-3;
  int session;

  if ((this_cd->track[track_no].type == BOOT) || (this_cd->track[track_no].type == HFS) || (block<18) || this_cd->single)  
    /* 18 sectors seems to be OK for ISO */ 

    result = this_cd->track[track_no].start_lba+block;

  else {
    
    /* ISO past sector 18 */
    result = block;
    
    /* post processing */
    for (session=0; session < this_cd->nr_iso_sessions; session++){
      PRINT("this is sector %d, checking session %d: %d-%d\n", result, session, 
            this_cd->lba_iso_sessions[session].start,
            this_cd->lba_iso_sessions[session].stop);
      if ((this_cd->lba_iso_sessions[session].start<=result)
          && (result<=this_cd->lba_iso_sessions[session].stop)) 
        goto exit;    /* OK, reading from an ISO session */
      if ((this_cd->lba_iso_sessions[session].stop<result)
          && (result<this_cd->lba_iso_sessions[session].start)) {
        result = 0;
        goto exit;    /* not OK, reading between two ISO sessions => we force 0*/
      }
    }

    /* We only get here if we read past the last session => we force 0 */
    result = 0;

  }

 exit: 

  PRINT("BMAP('%s', block %d) => sector %d\n", this_cd->track[inode->i_ino-3].name, block, result);

  return result;
}

/***************************************************************************/

struct file_operations cdfs_file_operations = {
  NULL,			/* lseek - default */
  generic_file_read,                                        /* read */
  NULL,                                                     /* write - bad */
  NULL,			/* readdir */
  NULL,			/* poll - default */
  NULL,			/* ioctl */
  NULL,			/* mmap */
  NULL,			/* open */
  NULL,			/* flush */
  NULL,			/* release */
  NULL,			/* fsync */
  NULL,			/* fasync */
  NULL,			/* check_media_change */
  NULL			/* revalidate */
};

struct inode_operations cdfs_file_inode_operations = {
  &cdfs_file_operations,
  NULL,			/* create */
  NULL,			/* lookup */
  NULL,			/* link */
  NULL,			/* unlink */
  NULL,			/* symlink */
  NULL,			/* mkdir */
  NULL,			/* rmdir */
  NULL,			/* mknod */
  NULL,			/* rename */
  NULL,			/* readlink */
  NULL,			/* follow_link */
  generic_readpage,	                                        /* readpage */
  NULL,			/* writepage */
  cdfs_bmap,		                    /* bmap */
  NULL,			/* truncate */
  NULL,			/* permission */
  NULL,			/* smap */
};

